# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast

import httpx

from .excludes import (
    ExcludesResource,
    AsyncExcludesResource,
    ExcludesResourceWithRawResponse,
    AsyncExcludesResourceWithRawResponse,
    ExcludesResourceWithStreamingResponse,
    AsyncExcludesResourceWithStreamingResponse,
)
from .includes import (
    IncludesResource,
    AsyncIncludesResource,
    IncludesResourceWithRawResponse,
    AsyncIncludesResourceWithRawResponse,
    IncludesResourceWithStreamingResponse,
    AsyncIncludesResourceWithStreamingResponse,
)
from ......_types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ......_utils import path_template, maybe_transform, async_maybe_transform
from ......_compat import cached_property
from .certificates import (
    CertificatesResource,
    AsyncCertificatesResource,
    CertificatesResourceWithRawResponse,
    AsyncCertificatesResourceWithRawResponse,
    CertificatesResourceWithStreamingResponse,
    AsyncCertificatesResourceWithStreamingResponse,
)
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from .fallback_domains import (
    FallbackDomainsResource,
    AsyncFallbackDomainsResource,
    FallbackDomainsResourceWithRawResponse,
    AsyncFallbackDomainsResourceWithRawResponse,
    FallbackDomainsResourceWithStreamingResponse,
    AsyncFallbackDomainsResourceWithStreamingResponse,
)
from ......_base_client import make_request_options
from ......types.zero_trust.devices.policies import default_edit_params
from ......types.zero_trust.devices.split_tunnel_exclude_param import SplitTunnelExcludeParam
from ......types.zero_trust.devices.split_tunnel_include_param import SplitTunnelIncludeParam
from ......types.zero_trust.devices.policies.default_get_response import DefaultGetResponse
from ......types.zero_trust.devices.policies.default_edit_response import DefaultEditResponse

__all__ = ["DefaultResource", "AsyncDefaultResource"]


class DefaultResource(SyncAPIResource):
    @cached_property
    def excludes(self) -> ExcludesResource:
        return ExcludesResource(self._client)

    @cached_property
    def includes(self) -> IncludesResource:
        return IncludesResource(self._client)

    @cached_property
    def fallback_domains(self) -> FallbackDomainsResource:
        return FallbackDomainsResource(self._client)

    @cached_property
    def certificates(self) -> CertificatesResource:
        return CertificatesResource(self._client)

    @cached_property
    def with_raw_response(self) -> DefaultResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DefaultResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DefaultResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DefaultResourceWithStreamingResponse(self)

    def edit(
        self,
        *,
        account_id: str,
        allow_mode_switch: bool | Omit = omit,
        allow_updates: bool | Omit = omit,
        allowed_to_leave: bool | Omit = omit,
        auto_connect: float | Omit = omit,
        captive_portal: float | Omit = omit,
        disable_auto_fallback: bool | Omit = omit,
        dns_search_suffixes: Iterable[default_edit_params.DNSSearchSuffix] | Omit = omit,
        exclude: Iterable[SplitTunnelExcludeParam] | Omit = omit,
        exclude_office_ips: bool | Omit = omit,
        global_acceleration: Optional[default_edit_params.GlobalAcceleration] | Omit = omit,
        include: Iterable[SplitTunnelIncludeParam] | Omit = omit,
        lan_allow_minutes: float | Omit = omit,
        lan_allow_subnet_size: float | Omit = omit,
        register_interface_ip_with_dns: bool | Omit = omit,
        sccm_vpn_boundary_support: bool | Omit = omit,
        service_mode_v2: default_edit_params.ServiceModeV2 | Omit = omit,
        support_url: str | Omit = omit,
        switch_locked: bool | Omit = omit,
        tunnel_protocol: str | Omit = omit,
        virtual_networks: Optional[default_edit_params.VirtualNetworks] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DefaultEditResponse]:
        """
        Updates the default device settings profile for an account.

        Args:
          allow_mode_switch: Whether to allow the user to switch WARP between modes.

          allow_updates: Whether to receive update notifications when a new version of the client is
              available.

          allowed_to_leave: Whether to allow devices to leave the organization.

          auto_connect: The amount of time in seconds to reconnect after having been disabled.

          captive_portal: Turn on the captive portal after the specified amount of time.

          disable_auto_fallback: If the `dns_server` field of a fallback domain is not present, the client will
              fall back to a best guess of the default/system DNS resolvers unless this policy
              option is set to `true`.

          dns_search_suffixes: List of DNS search suffixes to apply to clients. Suffixes are evaluated in
              order. Use an empty array to clear.

          exclude: List of routes excluded in the WARP client's tunnel. Both 'exclude' and
              'include' cannot be set in the same request.

          exclude_office_ips: Whether to add Microsoft IPs to Split Tunnel exclusions.

          global_acceleration: Global Acceleration settings for China. When configured, WARP clients connect to
              the Global Accelerator addresses instead of the default ones. Please contact
              your account representative to enable this feature on your account. See
              https://developers.cloudflare.com/china-network/concepts/global-acceleration/.

          include: List of routes included in the WARP client's tunnel. Both 'exclude' and
              'include' cannot be set in the same request.

          lan_allow_minutes: The amount of time in minutes a user is allowed access to their LAN. A value of
              0 will allow LAN access until the next WARP reconnection, such as a reboot or a
              laptop waking from sleep. Note that this field is omitted from the response if
              null or unset.

          lan_allow_subnet_size: The size of the subnet for the local access network. Note that this field is
              omitted from the response if null or unset.

          register_interface_ip_with_dns: Determines if the operating system will register WARP's local interface IP with
              your on-premises DNS server.

          sccm_vpn_boundary_support: Determines whether the WARP client indicates to SCCM that it is inside a VPN
              boundary. (Windows only).

          support_url: The URL to launch when the Send Feedback button is clicked.

          switch_locked: Whether to allow the user to turn off the WARP switch and disconnect the client.

          tunnel_protocol: Determines which tunnel protocol to use.

          virtual_networks: Virtual network access settings for the device.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._patch(
            path_template("/accounts/{account_id}/devices/policy", account_id=account_id),
            body=maybe_transform(
                {
                    "allow_mode_switch": allow_mode_switch,
                    "allow_updates": allow_updates,
                    "allowed_to_leave": allowed_to_leave,
                    "auto_connect": auto_connect,
                    "captive_portal": captive_portal,
                    "disable_auto_fallback": disable_auto_fallback,
                    "dns_search_suffixes": dns_search_suffixes,
                    "exclude": exclude,
                    "exclude_office_ips": exclude_office_ips,
                    "global_acceleration": global_acceleration,
                    "include": include,
                    "lan_allow_minutes": lan_allow_minutes,
                    "lan_allow_subnet_size": lan_allow_subnet_size,
                    "register_interface_ip_with_dns": register_interface_ip_with_dns,
                    "sccm_vpn_boundary_support": sccm_vpn_boundary_support,
                    "service_mode_v2": service_mode_v2,
                    "support_url": support_url,
                    "switch_locked": switch_locked,
                    "tunnel_protocol": tunnel_protocol,
                    "virtual_networks": virtual_networks,
                },
                default_edit_params.DefaultEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DefaultEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DefaultEditResponse]], ResultWrapper[DefaultEditResponse]),
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DefaultGetResponse]:
        """
        Fetches the default device settings profile for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/devices/policy", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DefaultGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DefaultGetResponse]], ResultWrapper[DefaultGetResponse]),
        )


class AsyncDefaultResource(AsyncAPIResource):
    @cached_property
    def excludes(self) -> AsyncExcludesResource:
        return AsyncExcludesResource(self._client)

    @cached_property
    def includes(self) -> AsyncIncludesResource:
        return AsyncIncludesResource(self._client)

    @cached_property
    def fallback_domains(self) -> AsyncFallbackDomainsResource:
        return AsyncFallbackDomainsResource(self._client)

    @cached_property
    def certificates(self) -> AsyncCertificatesResource:
        return AsyncCertificatesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDefaultResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDefaultResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDefaultResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDefaultResourceWithStreamingResponse(self)

    async def edit(
        self,
        *,
        account_id: str,
        allow_mode_switch: bool | Omit = omit,
        allow_updates: bool | Omit = omit,
        allowed_to_leave: bool | Omit = omit,
        auto_connect: float | Omit = omit,
        captive_portal: float | Omit = omit,
        disable_auto_fallback: bool | Omit = omit,
        dns_search_suffixes: Iterable[default_edit_params.DNSSearchSuffix] | Omit = omit,
        exclude: Iterable[SplitTunnelExcludeParam] | Omit = omit,
        exclude_office_ips: bool | Omit = omit,
        global_acceleration: Optional[default_edit_params.GlobalAcceleration] | Omit = omit,
        include: Iterable[SplitTunnelIncludeParam] | Omit = omit,
        lan_allow_minutes: float | Omit = omit,
        lan_allow_subnet_size: float | Omit = omit,
        register_interface_ip_with_dns: bool | Omit = omit,
        sccm_vpn_boundary_support: bool | Omit = omit,
        service_mode_v2: default_edit_params.ServiceModeV2 | Omit = omit,
        support_url: str | Omit = omit,
        switch_locked: bool | Omit = omit,
        tunnel_protocol: str | Omit = omit,
        virtual_networks: Optional[default_edit_params.VirtualNetworks] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DefaultEditResponse]:
        """
        Updates the default device settings profile for an account.

        Args:
          allow_mode_switch: Whether to allow the user to switch WARP between modes.

          allow_updates: Whether to receive update notifications when a new version of the client is
              available.

          allowed_to_leave: Whether to allow devices to leave the organization.

          auto_connect: The amount of time in seconds to reconnect after having been disabled.

          captive_portal: Turn on the captive portal after the specified amount of time.

          disable_auto_fallback: If the `dns_server` field of a fallback domain is not present, the client will
              fall back to a best guess of the default/system DNS resolvers unless this policy
              option is set to `true`.

          dns_search_suffixes: List of DNS search suffixes to apply to clients. Suffixes are evaluated in
              order. Use an empty array to clear.

          exclude: List of routes excluded in the WARP client's tunnel. Both 'exclude' and
              'include' cannot be set in the same request.

          exclude_office_ips: Whether to add Microsoft IPs to Split Tunnel exclusions.

          global_acceleration: Global Acceleration settings for China. When configured, WARP clients connect to
              the Global Accelerator addresses instead of the default ones. Please contact
              your account representative to enable this feature on your account. See
              https://developers.cloudflare.com/china-network/concepts/global-acceleration/.

          include: List of routes included in the WARP client's tunnel. Both 'exclude' and
              'include' cannot be set in the same request.

          lan_allow_minutes: The amount of time in minutes a user is allowed access to their LAN. A value of
              0 will allow LAN access until the next WARP reconnection, such as a reboot or a
              laptop waking from sleep. Note that this field is omitted from the response if
              null or unset.

          lan_allow_subnet_size: The size of the subnet for the local access network. Note that this field is
              omitted from the response if null or unset.

          register_interface_ip_with_dns: Determines if the operating system will register WARP's local interface IP with
              your on-premises DNS server.

          sccm_vpn_boundary_support: Determines whether the WARP client indicates to SCCM that it is inside a VPN
              boundary. (Windows only).

          support_url: The URL to launch when the Send Feedback button is clicked.

          switch_locked: Whether to allow the user to turn off the WARP switch and disconnect the client.

          tunnel_protocol: Determines which tunnel protocol to use.

          virtual_networks: Virtual network access settings for the device.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._patch(
            path_template("/accounts/{account_id}/devices/policy", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "allow_mode_switch": allow_mode_switch,
                    "allow_updates": allow_updates,
                    "allowed_to_leave": allowed_to_leave,
                    "auto_connect": auto_connect,
                    "captive_portal": captive_portal,
                    "disable_auto_fallback": disable_auto_fallback,
                    "dns_search_suffixes": dns_search_suffixes,
                    "exclude": exclude,
                    "exclude_office_ips": exclude_office_ips,
                    "global_acceleration": global_acceleration,
                    "include": include,
                    "lan_allow_minutes": lan_allow_minutes,
                    "lan_allow_subnet_size": lan_allow_subnet_size,
                    "register_interface_ip_with_dns": register_interface_ip_with_dns,
                    "sccm_vpn_boundary_support": sccm_vpn_boundary_support,
                    "service_mode_v2": service_mode_v2,
                    "support_url": support_url,
                    "switch_locked": switch_locked,
                    "tunnel_protocol": tunnel_protocol,
                    "virtual_networks": virtual_networks,
                },
                default_edit_params.DefaultEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DefaultEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DefaultEditResponse]], ResultWrapper[DefaultEditResponse]),
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DefaultGetResponse]:
        """
        Fetches the default device settings profile for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/devices/policy", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DefaultGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DefaultGetResponse]], ResultWrapper[DefaultGetResponse]),
        )


class DefaultResourceWithRawResponse:
    def __init__(self, default: DefaultResource) -> None:
        self._default = default

        self.edit = to_raw_response_wrapper(
            default.edit,
        )
        self.get = to_raw_response_wrapper(
            default.get,
        )

    @cached_property
    def excludes(self) -> ExcludesResourceWithRawResponse:
        return ExcludesResourceWithRawResponse(self._default.excludes)

    @cached_property
    def includes(self) -> IncludesResourceWithRawResponse:
        return IncludesResourceWithRawResponse(self._default.includes)

    @cached_property
    def fallback_domains(self) -> FallbackDomainsResourceWithRawResponse:
        return FallbackDomainsResourceWithRawResponse(self._default.fallback_domains)

    @cached_property
    def certificates(self) -> CertificatesResourceWithRawResponse:
        return CertificatesResourceWithRawResponse(self._default.certificates)


class AsyncDefaultResourceWithRawResponse:
    def __init__(self, default: AsyncDefaultResource) -> None:
        self._default = default

        self.edit = async_to_raw_response_wrapper(
            default.edit,
        )
        self.get = async_to_raw_response_wrapper(
            default.get,
        )

    @cached_property
    def excludes(self) -> AsyncExcludesResourceWithRawResponse:
        return AsyncExcludesResourceWithRawResponse(self._default.excludes)

    @cached_property
    def includes(self) -> AsyncIncludesResourceWithRawResponse:
        return AsyncIncludesResourceWithRawResponse(self._default.includes)

    @cached_property
    def fallback_domains(self) -> AsyncFallbackDomainsResourceWithRawResponse:
        return AsyncFallbackDomainsResourceWithRawResponse(self._default.fallback_domains)

    @cached_property
    def certificates(self) -> AsyncCertificatesResourceWithRawResponse:
        return AsyncCertificatesResourceWithRawResponse(self._default.certificates)


class DefaultResourceWithStreamingResponse:
    def __init__(self, default: DefaultResource) -> None:
        self._default = default

        self.edit = to_streamed_response_wrapper(
            default.edit,
        )
        self.get = to_streamed_response_wrapper(
            default.get,
        )

    @cached_property
    def excludes(self) -> ExcludesResourceWithStreamingResponse:
        return ExcludesResourceWithStreamingResponse(self._default.excludes)

    @cached_property
    def includes(self) -> IncludesResourceWithStreamingResponse:
        return IncludesResourceWithStreamingResponse(self._default.includes)

    @cached_property
    def fallback_domains(self) -> FallbackDomainsResourceWithStreamingResponse:
        return FallbackDomainsResourceWithStreamingResponse(self._default.fallback_domains)

    @cached_property
    def certificates(self) -> CertificatesResourceWithStreamingResponse:
        return CertificatesResourceWithStreamingResponse(self._default.certificates)


class AsyncDefaultResourceWithStreamingResponse:
    def __init__(self, default: AsyncDefaultResource) -> None:
        self._default = default

        self.edit = async_to_streamed_response_wrapper(
            default.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            default.get,
        )

    @cached_property
    def excludes(self) -> AsyncExcludesResourceWithStreamingResponse:
        return AsyncExcludesResourceWithStreamingResponse(self._default.excludes)

    @cached_property
    def includes(self) -> AsyncIncludesResourceWithStreamingResponse:
        return AsyncIncludesResourceWithStreamingResponse(self._default.includes)

    @cached_property
    def fallback_domains(self) -> AsyncFallbackDomainsResourceWithStreamingResponse:
        return AsyncFallbackDomainsResourceWithStreamingResponse(self._default.fallback_domains)

    @cached_property
    def certificates(self) -> AsyncCertificatesResourceWithStreamingResponse:
        return AsyncCertificatesResourceWithStreamingResponse(self._default.certificates)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["IdPFederationGrant"]


class IdPFederationGrant(BaseModel):
    id: str
    """UID of the IdP federation grant."""

    idp_id: str
    """UID of the identity provider being federated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["AppGetResponse", "Data", "Paging"]


class Data(BaseModel):
    id: Optional[str] = None

    created_at: Optional[datetime] = None

    name: Optional[str] = None


class Paging(BaseModel):
    end_offset: Optional[float] = None

    start_offset: Optional[float] = None

    total_count: Optional[float] = None


class AppGetResponse(BaseModel):
    data: Optional[List[Data]] = None

    paging: Optional[Paging] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os

import httpx

from ...._files import read_file_content
from ...._types import (
    Body,
    Omit,
    Query,
    Headers,
    NotGiven,
    BinaryTypes,
    FileContent,
    omit,
    not_given,
)
from ...._utils import path_template
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.logs.log_explorer.query_sql_response import QuerySqlResponse

__all__ = ["QueryResource", "AsyncQueryResource"]


class QueryResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> QueryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return QueryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QueryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return QueryResourceWithStreamingResponse(self)

    def sql(
        self,
        body: FileContent | BinaryTypes,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[QuerySqlResponse]:
        """
        Run a SQL query against account or zone-level datasets.

        Timestamp fields are RFC3339 strings. Filter with: WHERE {timestamp_field} >=
        now() - INTERVAL '30' DAY WHERE {timestamp_field} >= '2026-04-01T00:00:00Z'
        WHERE {timestamp_field} BETWEEN '2026-04-01T00:00:00Z' AND
        '2026-04-30T23:59:59Z'

        Check /account or zones/{account or zone_id}/logs/explorer/datasets to see
        enabled account or zone level datasets. Zone-level datasets will not appear
        here. Check /account or zones/{account or
        zone_id}/logs/explorer/datasets/available for the schemas, and the name of the
        timestamp fields.

        For zone-level datasets use the zone-scoped endpoint: POST
        /zones/{zone_id}/logs/explorer/query/sql

        For more information about the datasets, and the meaning of each field, check
        out https://developers.cloudflare.com/logs/logpush/logpush-job/datasets/

        Args:
          body: SQL query to execute.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        extra_headers = {"Content-Type": "text/plain", **(extra_headers or {})}
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logs/explorer/query/sql",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncSinglePage[QuerySqlResponse],
            content=read_file_content(body) if isinstance(body, os.PathLike) else body,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=QuerySqlResponse,
            method="post",
        )


class AsyncQueryResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncQueryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncQueryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQueryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncQueryResourceWithStreamingResponse(self)

    def sql(
        self,
        body: FileContent | BinaryTypes,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[QuerySqlResponse, AsyncSinglePage[QuerySqlResponse]]:
        """
        Run a SQL query against account or zone-level datasets.

        Timestamp fields are RFC3339 strings. Filter with: WHERE {timestamp_field} >=
        now() - INTERVAL '30' DAY WHERE {timestamp_field} >= '2026-04-01T00:00:00Z'
        WHERE {timestamp_field} BETWEEN '2026-04-01T00:00:00Z' AND
        '2026-04-30T23:59:59Z'

        Check /account or zones/{account or zone_id}/logs/explorer/datasets to see
        enabled account or zone level datasets. Zone-level datasets will not appear
        here. Check /account or zones/{account or
        zone_id}/logs/explorer/datasets/available for the schemas, and the name of the
        timestamp fields.

        For zone-level datasets use the zone-scoped endpoint: POST
        /zones/{zone_id}/logs/explorer/query/sql

        For more information about the datasets, and the meaning of each field, check
        out https://developers.cloudflare.com/logs/logpush/logpush-job/datasets/

        Args:
          body: SQL query to execute.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        extra_headers = {"Content-Type": "text/plain", **(extra_headers or {})}
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logs/explorer/query/sql",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncSinglePage[QuerySqlResponse],
            content=read_file_content(body) if isinstance(body, os.PathLike) else body,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=QuerySqlResponse,
            method="post",
        )


class QueryResourceWithRawResponse:
    def __init__(self, query: QueryResource) -> None:
        self._query = query

        self.sql = to_raw_response_wrapper(
            query.sql,
        )


class AsyncQueryResourceWithRawResponse:
    def __init__(self, query: AsyncQueryResource) -> None:
        self._query = query

        self.sql = async_to_raw_response_wrapper(
            query.sql,
        )


class QueryResourceWithStreamingResponse:
    def __init__(self, query: QueryResource) -> None:
        self._query = query

        self.sql = to_streamed_response_wrapper(
            query.sql,
        )


class AsyncQueryResourceWithStreamingResponse:
    def __init__(self, query: AsyncQueryResource) -> None:
        self._query = query

        self.sql = async_to_streamed_response_wrapper(
            query.sql,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel
from ..devices.schema_data import SchemaData

__all__ = ["RuleCreateResponse", "TargetedTest"]


class TargetedTest(BaseModel):
    data: SchemaData
    """
    The configuration object which contains the details for the WARP client to
    conduct the test.
    """

    enabled: bool

    name: str

    test_id: str


class RuleCreateResponse(BaseModel):
    id: str
    """API Resource UUID tag."""

    created_at: str

    match: str

    name: str

    description: Optional[str] = None

    targeted_tests: Optional[List[TargetedTest]] = None

    updated_at: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["FlagDeleteResponse"]


class FlagDeleteResponse(BaseModel):
    key: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["IntegrationListParams"]


class IntegrationListParams(TypedDict, total=False):
    account_id: Required[str]

    application: str
    """Filter by application/vendor (e.g., GOOGLE_WORKSPACE, MICROSOFT_INTERNAL)."""

    direction: Literal["asc", "desc"]
    """Direction to order results."""

    dlp_enabled: bool
    """Filter by DLP enabled status (true/false)."""

    order: Literal["application", "created", "name", "status"]
    """Field to order results by."""

    page: int
    """Page number within the paginated result set."""

    page_size: int
    """Number of results per page."""

    search: str
    """Search integrations by name or application."""

    status: Literal["Healthy", "Initializing", "Offline", "Unhealthy"]
    """Filter by integration status."""

    use_cases: str
    """Filter by enabled use cases (e.g., casb, ces).

    Matches integrations enrolled in any of the specified values. Can be specified
    multiple times.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.intel.domains import bulk_get_params
from ....types.intel.domains.bulk_get_response import BulkGetResponse

__all__ = ["BulksResource", "AsyncBulksResource"]


class BulksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BulksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return BulksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BulksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return BulksResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        account_id: str,
        domain: SequenceNotStr[str] | Omit = omit,
        include_ranking: bool | Omit = omit,
        skip_ranking: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BulkGetResponse]:
        """
        Returns security details and statistics about multiple domains in a single
        request.

        **Behavior change — domain ranking is becoming opt-in.** This endpoint
        previously included domain ranking data in every response and accepted a
        `skip_ranking=true` query parameter to opt out. That parameter is being
        deprecated and ranking will no longer be returned by default. Callers that want
        ranking data must pass `include_ranking=true`. The `skip_ranking` parameter will
        be silently ignored once the change ships.

        Args:
          account_id: Identifier.

          domain: Accepts multiple values like `?domain=cloudflare.com&domain=example.com`.

          include_ranking: Whether to include domain ranking data in the response. Defaults to `false` —
              ranking lookups are expensive at bulk scale and most callers do not need them.
              Set to `true` to opt in. This parameter replaces the deprecated `skip_ranking`
              (see below).

          skip_ranking: **Deprecated.** Previously controlled whether the ranking lookup was skipped
              (defaulted to `false`, meaning ranking ran). The endpoint's default behavior is
              being flipped — ranking is now opt-in via `include_ranking=true` — and this
              parameter will be silently ignored. Remove it from your callers and use
              `include_ranking` instead.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/intel/domain/bulk", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "domain": domain,
                        "include_ranking": include_ranking,
                        "skip_ranking": skip_ranking,
                    },
                    bulk_get_params.BulkGetParams,
                ),
                post_parser=ResultWrapper[Optional[BulkGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[BulkGetResponse]], ResultWrapper[BulkGetResponse]),
        )


class AsyncBulksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBulksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBulksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBulksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncBulksResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        account_id: str,
        domain: SequenceNotStr[str] | Omit = omit,
        include_ranking: bool | Omit = omit,
        skip_ranking: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BulkGetResponse]:
        """
        Returns security details and statistics about multiple domains in a single
        request.

        **Behavior change — domain ranking is becoming opt-in.** This endpoint
        previously included domain ranking data in every response and accepted a
        `skip_ranking=true` query parameter to opt out. That parameter is being
        deprecated and ranking will no longer be returned by default. Callers that want
        ranking data must pass `include_ranking=true`. The `skip_ranking` parameter will
        be silently ignored once the change ships.

        Args:
          account_id: Identifier.

          domain: Accepts multiple values like `?domain=cloudflare.com&domain=example.com`.

          include_ranking: Whether to include domain ranking data in the response. Defaults to `false` —
              ranking lookups are expensive at bulk scale and most callers do not need them.
              Set to `true` to opt in. This parameter replaces the deprecated `skip_ranking`
              (see below).

          skip_ranking: **Deprecated.** Previously controlled whether the ranking lookup was skipped
              (defaulted to `false`, meaning ranking ran). The endpoint's default behavior is
              being flipped — ranking is now opt-in via `include_ranking=true` — and this
              parameter will be silently ignored. Remove it from your callers and use
              `include_ranking` instead.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/intel/domain/bulk", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "domain": domain,
                        "include_ranking": include_ranking,
                        "skip_ranking": skip_ranking,
                    },
                    bulk_get_params.BulkGetParams,
                ),
                post_parser=ResultWrapper[Optional[BulkGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[BulkGetResponse]], ResultWrapper[BulkGetResponse]),
        )


class BulksResourceWithRawResponse:
    def __init__(self, bulks: BulksResource) -> None:
        self._bulks = bulks

        self.get = to_raw_response_wrapper(
            bulks.get,
        )


class AsyncBulksResourceWithRawResponse:
    def __init__(self, bulks: AsyncBulksResource) -> None:
        self._bulks = bulks

        self.get = async_to_raw_response_wrapper(
            bulks.get,
        )


class BulksResourceWithStreamingResponse:
    def __init__(self, bulks: BulksResource) -> None:
        self._bulks = bulks

        self.get = to_streamed_response_wrapper(
            bulks.get,
        )


class AsyncBulksResourceWithStreamingResponse:
    def __init__(self, bulks: AsyncBulksResource) -> None:
        self._bulks = bulks

        self.get = async_to_streamed_response_wrapper(
            bulks.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, Optional, cast
from datetime import datetime

import httpx

from .tokens import (
    TokensResource,
    AsyncTokensResource,
    TokensResourceWithRawResponse,
    AsyncTokensResourceWithRawResponse,
    TokensResourceWithStreamingResponse,
    AsyncTokensResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....types.moq import relay_list_params, relay_create_params, relay_update_params
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.moq.relay_get_response import RelayGetResponse
from ....types.moq.relay_list_response import RelayListResponse
from ....types.moq.relay_create_response import RelayCreateResponse
from ....types.moq.relay_update_response import RelayUpdateResponse

__all__ = ["RelaysResource", "AsyncRelaysResource"]


class RelaysResource(SyncAPIResource):
    @cached_property
    def tokens(self) -> TokensResource:
        return TokensResource(self._client)

    @cached_property
    def with_raw_response(self) -> RelaysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RelaysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RelaysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RelaysResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RelayCreateResponse]:
        """Provisions a new MoQ relay instance.

        Auto-creates a publish+subscribe token and
        a subscribe-only token. Token values are included in the response (shown once).
        Config is set to defaults (lingering subscribe enabled, 30s ceiling, upstreams
        off). Use PUT to modify.

        Args:
          account_id: Cloudflare account identifier.

          name: Human-readable name for the relay.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/moq/relays", account_id=account_id),
            body=maybe_transform({"name": name}, relay_create_params.RelayCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RelayCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RelayCreateResponse]], ResultWrapper[RelayCreateResponse]),
        )

    def update(
        self,
        relay_id: str,
        *,
        account_id: str,
        config: relay_update_params.Config | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RelayUpdateResponse]:
        """Updates a relay's name and/or configuration.

        Partial updates: omitted fields are
        preserved. Config sub-objects replace as whole objects when present. upstreams
        and lingering_subscribe are mutually exclusive.

        Args:
          account_id: Cloudflare account identifier.

          config: upstreams and lingering_subscribe are mutually exclusive.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/moq/relays/{relay_id}", account_id=account_id, relay_id=relay_id),
            body=maybe_transform(
                {
                    "config": config,
                    "name": name,
                },
                relay_update_params.RelayUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RelayUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RelayUpdateResponse]], ResultWrapper[RelayUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        asc: bool | Omit = omit,
        created_after: Union[str, datetime] | Omit = omit,
        created_before: Union[str, datetime] | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[RelayListResponse]:
        """Lists all MoQ relays for the account.

        Returns only metadata. Config, status, and
        tokens are omitted.

        Results are cursor-paginated (keyset on the `created` timestamp). Use
        `created_before` / `created_after` with the `created` value of the first/last
        item in a page to fetch the adjacent page. `result_info` reports the page
        `count` and the `total` matching the cursor filters.

        Args:
          account_id: Cloudflare account identifier.

          asc: Sort order by `created`. When true, results are returned oldest-first
              (ascending); otherwise newest-first (descending, the default).

          created_after: Cursor for pagination. Returns relays created strictly after this RFC 3339
              timestamp (typically the `created` value of the last item on the current page,
              to fetch the next page).

          created_before: Cursor for pagination. Returns relays created strictly before this RFC 3339
              timestamp (typically the `created` value of the first item on the current page,
              to fetch the previous page).

          per_page: Maximum number of relays to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/moq/relays", account_id=account_id),
            page=SyncSinglePage[RelayListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asc": asc,
                        "created_after": created_after,
                        "created_before": created_before,
                        "per_page": per_page,
                    },
                    relay_list_params.RelayListParams,
                ),
            ),
            model=RelayListResponse,
        )

    def delete(
        self,
        relay_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Soft-deletes a MoQ relay.

        Args:
          account_id: Cloudflare account identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/moq/relays/{relay_id}", account_id=account_id, relay_id=relay_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        relay_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RelayGetResponse]:
        """Retrieves a single MoQ relay including config and status.

        Tokens are NOT
        included.

        Args:
          account_id: Cloudflare account identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/moq/relays/{relay_id}", account_id=account_id, relay_id=relay_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RelayGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RelayGetResponse]], ResultWrapper[RelayGetResponse]),
        )


class AsyncRelaysResource(AsyncAPIResource):
    @cached_property
    def tokens(self) -> AsyncTokensResource:
        return AsyncTokensResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRelaysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRelaysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRelaysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRelaysResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RelayCreateResponse]:
        """Provisions a new MoQ relay instance.

        Auto-creates a publish+subscribe token and
        a subscribe-only token. Token values are included in the response (shown once).
        Config is set to defaults (lingering subscribe enabled, 30s ceiling, upstreams
        off). Use PUT to modify.

        Args:
          account_id: Cloudflare account identifier.

          name: Human-readable name for the relay.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/moq/relays", account_id=account_id),
            body=await async_maybe_transform({"name": name}, relay_create_params.RelayCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RelayCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RelayCreateResponse]], ResultWrapper[RelayCreateResponse]),
        )

    async def update(
        self,
        relay_id: str,
        *,
        account_id: str,
        config: relay_update_params.Config | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RelayUpdateResponse]:
        """Updates a relay's name and/or configuration.

        Partial updates: omitted fields are
        preserved. Config sub-objects replace as whole objects when present. upstreams
        and lingering_subscribe are mutually exclusive.

        Args:
          account_id: Cloudflare account identifier.

          config: upstreams and lingering_subscribe are mutually exclusive.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/moq/relays/{relay_id}", account_id=account_id, relay_id=relay_id),
            body=await async_maybe_transform(
                {
                    "config": config,
                    "name": name,
                },
                relay_update_params.RelayUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RelayUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RelayUpdateResponse]], ResultWrapper[RelayUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        asc: bool | Omit = omit,
        created_after: Union[str, datetime] | Omit = omit,
        created_before: Union[str, datetime] | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RelayListResponse, AsyncSinglePage[RelayListResponse]]:
        """Lists all MoQ relays for the account.

        Returns only metadata. Config, status, and
        tokens are omitted.

        Results are cursor-paginated (keyset on the `created` timestamp). Use
        `created_before` / `created_after` with the `created` value of the first/last
        item in a page to fetch the adjacent page. `result_info` reports the page
        `count` and the `total` matching the cursor filters.

        Args:
          account_id: Cloudflare account identifier.

          asc: Sort order by `created`. When true, results are returned oldest-first
              (ascending); otherwise newest-first (descending, the default).

          created_after: Cursor for pagination. Returns relays created strictly after this RFC 3339
              timestamp (typically the `created` value of the last item on the current page,
              to fetch the next page).

          created_before: Cursor for pagination. Returns relays created strictly before this RFC 3339
              timestamp (typically the `created` value of the first item on the current page,
              to fetch the previous page).

          per_page: Maximum number of relays to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/moq/relays", account_id=account_id),
            page=AsyncSinglePage[RelayListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asc": asc,
                        "created_after": created_after,
                        "created_before": created_before,
                        "per_page": per_page,
                    },
                    relay_list_params.RelayListParams,
                ),
            ),
            model=RelayListResponse,
        )

    async def delete(
        self,
        relay_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Soft-deletes a MoQ relay.

        Args:
          account_id: Cloudflare account identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/moq/relays/{relay_id}", account_id=account_id, relay_id=relay_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        relay_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RelayGetResponse]:
        """Retrieves a single MoQ relay including config and status.

        Tokens are NOT
        included.

        Args:
          account_id: Cloudflare account identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/moq/relays/{relay_id}", account_id=account_id, relay_id=relay_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RelayGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RelayGetResponse]], ResultWrapper[RelayGetResponse]),
        )


class RelaysResourceWithRawResponse:
    def __init__(self, relays: RelaysResource) -> None:
        self._relays = relays

        self.create = to_raw_response_wrapper(
            relays.create,
        )
        self.update = to_raw_response_wrapper(
            relays.update,
        )
        self.list = to_raw_response_wrapper(
            relays.list,
        )
        self.delete = to_raw_response_wrapper(
            relays.delete,
        )
        self.get = to_raw_response_wrapper(
            relays.get,
        )

    @cached_property
    def tokens(self) -> TokensResourceWithRawResponse:
        return TokensResourceWithRawResponse(self._relays.tokens)


class AsyncRelaysResourceWithRawResponse:
    def __init__(self, relays: AsyncRelaysResource) -> None:
        self._relays = relays

        self.create = async_to_raw_response_wrapper(
            relays.create,
        )
        self.update = async_to_raw_response_wrapper(
            relays.update,
        )
        self.list = async_to_raw_response_wrapper(
            relays.list,
        )
        self.delete = async_to_raw_response_wrapper(
            relays.delete,
        )
        self.get = async_to_raw_response_wrapper(
            relays.get,
        )

    @cached_property
    def tokens(self) -> AsyncTokensResourceWithRawResponse:
        return AsyncTokensResourceWithRawResponse(self._relays.tokens)


class RelaysResourceWithStreamingResponse:
    def __init__(self, relays: RelaysResource) -> None:
        self._relays = relays

        self.create = to_streamed_response_wrapper(
            relays.create,
        )
        self.update = to_streamed_response_wrapper(
            relays.update,
        )
        self.list = to_streamed_response_wrapper(
            relays.list,
        )
        self.delete = to_streamed_response_wrapper(
            relays.delete,
        )
        self.get = to_streamed_response_wrapper(
            relays.get,
        )

    @cached_property
    def tokens(self) -> TokensResourceWithStreamingResponse:
        return TokensResourceWithStreamingResponse(self._relays.tokens)


class AsyncRelaysResourceWithStreamingResponse:
    def __init__(self, relays: AsyncRelaysResource) -> None:
        self._relays = relays

        self.create = async_to_streamed_response_wrapper(
            relays.create,
        )
        self.update = async_to_streamed_response_wrapper(
            relays.update,
        )
        self.list = async_to_streamed_response_wrapper(
            relays.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            relays.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            relays.get,
        )

    @cached_property
    def tokens(self) -> AsyncTokensResourceWithStreamingResponse:
        return AsyncTokensResourceWithStreamingResponse(self._relays.tokens)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .data_tag_get_response import DataTagGetResponse as DataTagGetResponse
from .data_tag_create_params import DataTagCreateParams as DataTagCreateParams
from .data_tag_list_response import DataTagListResponse as DataTagListResponse
from .data_tag_update_params import DataTagUpdateParams as DataTagUpdateParams
from .data_tag_create_response import DataTagCreateResponse as DataTagCreateResponse
from .data_tag_update_response import DataTagUpdateResponse as DataTagUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["FlagListParams"]


class FlagListParams(TypedDict, total=False):
    account_id: Required[str]
    """Cloudflare account ID."""

    cursor: str
    """Pagination cursor from a previous response."""

    limit: str
    """Max items to return (1–200)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .zone import (
    ZoneResource,
    AsyncZoneResource,
    ZoneResourceWithRawResponse,
    AsyncZoneResourceWithRawResponse,
    ZoneResourceWithStreamingResponse,
    AsyncZoneResourceWithStreamingResponse,
)
from .account import (
    AccountResource,
    AsyncAccountResource,
    AccountResourceWithRawResponse,
    AsyncAccountResourceWithRawResponse,
    AccountResourceWithStreamingResponse,
    AsyncAccountResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["UsageResource", "AsyncUsageResource"]


class UsageResource(SyncAPIResource):
    @cached_property
    def zone(self) -> ZoneResource:
        return ZoneResource(self._client)

    @cached_property
    def account(self) -> AccountResource:
        return AccountResource(self._client)

    @cached_property
    def with_raw_response(self) -> UsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return UsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return UsageResourceWithStreamingResponse(self)


class AsyncUsageResource(AsyncAPIResource):
    @cached_property
    def zone(self) -> AsyncZoneResource:
        return AsyncZoneResource(self._client)

    @cached_property
    def account(self) -> AsyncAccountResource:
        return AsyncAccountResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncUsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncUsageResourceWithStreamingResponse(self)


class UsageResourceWithRawResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

    @cached_property
    def zone(self) -> ZoneResourceWithRawResponse:
        return ZoneResourceWithRawResponse(self._usage.zone)

    @cached_property
    def account(self) -> AccountResourceWithRawResponse:
        return AccountResourceWithRawResponse(self._usage.account)


class AsyncUsageResourceWithRawResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

    @cached_property
    def zone(self) -> AsyncZoneResourceWithRawResponse:
        return AsyncZoneResourceWithRawResponse(self._usage.zone)

    @cached_property
    def account(self) -> AsyncAccountResourceWithRawResponse:
        return AsyncAccountResourceWithRawResponse(self._usage.account)


class UsageResourceWithStreamingResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

    @cached_property
    def zone(self) -> ZoneResourceWithStreamingResponse:
        return ZoneResourceWithStreamingResponse(self._usage.zone)

    @cached_property
    def account(self) -> AccountResourceWithStreamingResponse:
        return AccountResourceWithStreamingResponse(self._usage.account)


class AsyncUsageResourceWithStreamingResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

    @cached_property
    def zone(self) -> AsyncZoneResourceWithStreamingResponse:
        return AsyncZoneResourceWithStreamingResponse(self._usage.zone)

    @cached_property
    def account(self) -> AsyncAccountResourceWithStreamingResponse:
        return AsyncAccountResourceWithStreamingResponse(self._usage.account)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .setup_flows import (
    SetupFlowsResource,
    AsyncSetupFlowsResource,
    SetupFlowsResourceWithRawResponse,
    AsyncSetupFlowsResourceWithRawResponse,
    SetupFlowsResourceWithStreamingResponse,
    AsyncSetupFlowsResourceWithStreamingResponse,
)
from .applications import (
    ApplicationsResource,
    AsyncApplicationsResource,
    ApplicationsResourceWithRawResponse,
    AsyncApplicationsResourceWithRawResponse,
    ApplicationsResourceWithStreamingResponse,
    AsyncApplicationsResourceWithStreamingResponse,
)

__all__ = [
    "SetupFlowsResource",
    "AsyncSetupFlowsResource",
    "SetupFlowsResourceWithRawResponse",
    "AsyncSetupFlowsResourceWithRawResponse",
    "SetupFlowsResourceWithStreamingResponse",
    "AsyncSetupFlowsResourceWithStreamingResponse",
    "ApplicationsResource",
    "AsyncApplicationsResource",
    "ApplicationsResourceWithRawResponse",
    "AsyncApplicationsResourceWithRawResponse",
    "ApplicationsResourceWithStreamingResponse",
    "AsyncApplicationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tenant_custom_nameserver_get_response import (
    TenantCustomNameserverGetResponse as TenantCustomNameserverGetResponse,
)
from .tenant_custom_nameserver_create_params import (
    TenantCustomNameserverCreateParams as TenantCustomNameserverCreateParams,
)
from .tenant_custom_nameserver_create_response import (
    TenantCustomNameserverCreateResponse as TenantCustomNameserverCreateResponse,
)
from .tenant_custom_nameserver_delete_response import (
    TenantCustomNameserverDeleteResponse as TenantCustomNameserverDeleteResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .prefix_bindings import (
    PrefixBindingsResource,
    AsyncPrefixBindingsResource,
    PrefixBindingsResourceWithRawResponse,
    AsyncPrefixBindingsResourceWithRawResponse,
    PrefixBindingsResourceWithStreamingResponse,
    AsyncPrefixBindingsResourceWithStreamingResponse,
)

__all__ = ["RegionalServicesResource", "AsyncRegionalServicesResource"]


class RegionalServicesResource(SyncAPIResource):
    @cached_property
    def prefix_bindings(self) -> PrefixBindingsResource:
        return PrefixBindingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> RegionalServicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RegionalServicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RegionalServicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RegionalServicesResourceWithStreamingResponse(self)


class AsyncRegionalServicesResource(AsyncAPIResource):
    @cached_property
    def prefix_bindings(self) -> AsyncPrefixBindingsResource:
        return AsyncPrefixBindingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRegionalServicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRegionalServicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRegionalServicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRegionalServicesResourceWithStreamingResponse(self)


class RegionalServicesResourceWithRawResponse:
    def __init__(self, regional_services: RegionalServicesResource) -> None:
        self._regional_services = regional_services

    @cached_property
    def prefix_bindings(self) -> PrefixBindingsResourceWithRawResponse:
        return PrefixBindingsResourceWithRawResponse(self._regional_services.prefix_bindings)


class AsyncRegionalServicesResourceWithRawResponse:
    def __init__(self, regional_services: AsyncRegionalServicesResource) -> None:
        self._regional_services = regional_services

    @cached_property
    def prefix_bindings(self) -> AsyncPrefixBindingsResourceWithRawResponse:
        return AsyncPrefixBindingsResourceWithRawResponse(self._regional_services.prefix_bindings)


class RegionalServicesResourceWithStreamingResponse:
    def __init__(self, regional_services: RegionalServicesResource) -> None:
        self._regional_services = regional_services

    @cached_property
    def prefix_bindings(self) -> PrefixBindingsResourceWithStreamingResponse:
        return PrefixBindingsResourceWithStreamingResponse(self._regional_services.prefix_bindings)


class AsyncRegionalServicesResourceWithStreamingResponse:
    def __init__(self, regional_services: AsyncRegionalServicesResource) -> None:
        self._regional_services = regional_services

    @cached_property
    def prefix_bindings(self) -> AsyncPrefixBindingsResourceWithStreamingResponse:
        return AsyncPrefixBindingsResourceWithStreamingResponse(self._regional_services.prefix_bindings)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from .summary import (
    SummaryResource,
    AsyncSummaryResource,
    SummaryResourceWithRawResponse,
    AsyncSummaryResourceWithRawResponse,
    SummaryResourceWithStreamingResponse,
    AsyncSummaryResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.radar.ai import inference_summary_v2_params, inference_timeseries_groups_v2_params
from .timeseries_groups.timeseries_groups import (
    TimeseriesGroupsResource,
    AsyncTimeseriesGroupsResource,
    TimeseriesGroupsResourceWithRawResponse,
    AsyncTimeseriesGroupsResourceWithRawResponse,
    TimeseriesGroupsResourceWithStreamingResponse,
    AsyncTimeseriesGroupsResourceWithStreamingResponse,
)
from .....types.radar.ai.inference_summary_v2_response import InferenceSummaryV2Response
from .....types.radar.ai.inference_timeseries_groups_v2_response import InferenceTimeseriesGroupsV2Response

__all__ = ["InferenceResource", "AsyncInferenceResource"]


class InferenceResource(SyncAPIResource):
    @cached_property
    def summary(self) -> SummaryResource:
        return SummaryResource(self._client)

    @cached_property
    def timeseries_groups(self) -> TimeseriesGroupsResource:
        return TimeseriesGroupsResource(self._client)

    @cached_property
    def with_raw_response(self) -> InferenceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return InferenceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InferenceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return InferenceResourceWithStreamingResponse(self)

    def summary_v2(
        self,
        dimension: Literal["MODEL", "TASK"],
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InferenceSummaryV2Response:
        """
        Retrieves an aggregated summary of the number of inferences run on Workers AI,
        grouped by the specified dimension.

        Args:
          dimension: Specifies the attribute by which to group the results.

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/ai/inference/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                    },
                    inference_summary_v2_params.InferenceSummaryV2Params,
                ),
                post_parser=ResultWrapper[InferenceSummaryV2Response]._unwrapper,
            ),
            cast_to=cast(Type[InferenceSummaryV2Response], ResultWrapper[InferenceSummaryV2Response]),
        )

    def timeseries_groups_v2(
        self,
        dimension: Literal["MODEL", "TASK"],
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        normalization: Literal["PERCENTAGE", "MIN0_MAX"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InferenceTimeseriesGroupsV2Response:
        """
        Retrieves the distribution of the number of inferences run on Workers AI,
        grouped by the specified dimension over time.

        Args:
          dimension: Specifies the attribute by which to group the results.

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          normalization: Normalization method applied to the results. Refer to
              [Normalization methods](https://developers.cloudflare.com/radar/concepts/normalization/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/ai/inference/timeseries_groups/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                        "normalization": normalization,
                    },
                    inference_timeseries_groups_v2_params.InferenceTimeseriesGroupsV2Params,
                ),
                post_parser=ResultWrapper[InferenceTimeseriesGroupsV2Response]._unwrapper,
            ),
            cast_to=cast(Type[InferenceTimeseriesGroupsV2Response], ResultWrapper[InferenceTimeseriesGroupsV2Response]),
        )


class AsyncInferenceResource(AsyncAPIResource):
    @cached_property
    def summary(self) -> AsyncSummaryResource:
        return AsyncSummaryResource(self._client)

    @cached_property
    def timeseries_groups(self) -> AsyncTimeseriesGroupsResource:
        return AsyncTimeseriesGroupsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncInferenceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInferenceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInferenceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncInferenceResourceWithStreamingResponse(self)

    async def summary_v2(
        self,
        dimension: Literal["MODEL", "TASK"],
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InferenceSummaryV2Response:
        """
        Retrieves an aggregated summary of the number of inferences run on Workers AI,
        grouped by the specified dimension.

        Args:
          dimension: Specifies the attribute by which to group the results.

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/ai/inference/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                    },
                    inference_summary_v2_params.InferenceSummaryV2Params,
                ),
                post_parser=ResultWrapper[InferenceSummaryV2Response]._unwrapper,
            ),
            cast_to=cast(Type[InferenceSummaryV2Response], ResultWrapper[InferenceSummaryV2Response]),
        )

    async def timeseries_groups_v2(
        self,
        dimension: Literal["MODEL", "TASK"],
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        normalization: Literal["PERCENTAGE", "MIN0_MAX"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InferenceTimeseriesGroupsV2Response:
        """
        Retrieves the distribution of the number of inferences run on Workers AI,
        grouped by the specified dimension over time.

        Args:
          dimension: Specifies the attribute by which to group the results.

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          normalization: Normalization method applied to the results. Refer to
              [Normalization methods](https://developers.cloudflare.com/radar/concepts/normalization/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/ai/inference/timeseries_groups/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                        "normalization": normalization,
                    },
                    inference_timeseries_groups_v2_params.InferenceTimeseriesGroupsV2Params,
                ),
                post_parser=ResultWrapper[InferenceTimeseriesGroupsV2Response]._unwrapper,
            ),
            cast_to=cast(Type[InferenceTimeseriesGroupsV2Response], ResultWrapper[InferenceTimeseriesGroupsV2Response]),
        )


class InferenceResourceWithRawResponse:
    def __init__(self, inference: InferenceResource) -> None:
        self._inference = inference

        self.summary_v2 = to_raw_response_wrapper(
            inference.summary_v2,
        )
        self.timeseries_groups_v2 = to_raw_response_wrapper(
            inference.timeseries_groups_v2,
        )

    @cached_property
    def summary(self) -> SummaryResourceWithRawResponse:
        return SummaryResourceWithRawResponse(self._inference.summary)

    @cached_property
    def timeseries_groups(self) -> TimeseriesGroupsResourceWithRawResponse:
        return TimeseriesGroupsResourceWithRawResponse(self._inference.timeseries_groups)


class AsyncInferenceResourceWithRawResponse:
    def __init__(self, inference: AsyncInferenceResource) -> None:
        self._inference = inference

        self.summary_v2 = async_to_raw_response_wrapper(
            inference.summary_v2,
        )
        self.timeseries_groups_v2 = async_to_raw_response_wrapper(
            inference.timeseries_groups_v2,
        )

    @cached_property
    def summary(self) -> AsyncSummaryResourceWithRawResponse:
        return AsyncSummaryResourceWithRawResponse(self._inference.summary)

    @cached_property
    def timeseries_groups(self) -> AsyncTimeseriesGroupsResourceWithRawResponse:
        return AsyncTimeseriesGroupsResourceWithRawResponse(self._inference.timeseries_groups)


class InferenceResourceWithStreamingResponse:
    def __init__(self, inference: InferenceResource) -> None:
        self._inference = inference

        self.summary_v2 = to_streamed_response_wrapper(
            inference.summary_v2,
        )
        self.timeseries_groups_v2 = to_streamed_response_wrapper(
            inference.timeseries_groups_v2,
        )

    @cached_property
    def summary(self) -> SummaryResourceWithStreamingResponse:
        return SummaryResourceWithStreamingResponse(self._inference.summary)

    @cached_property
    def timeseries_groups(self) -> TimeseriesGroupsResourceWithStreamingResponse:
        return TimeseriesGroupsResourceWithStreamingResponse(self._inference.timeseries_groups)


class AsyncInferenceResourceWithStreamingResponse:
    def __init__(self, inference: AsyncInferenceResource) -> None:
        self._inference = inference

        self.summary_v2 = async_to_streamed_response_wrapper(
            inference.summary_v2,
        )
        self.timeseries_groups_v2 = async_to_streamed_response_wrapper(
            inference.timeseries_groups_v2,
        )

    @cached_property
    def summary(self) -> AsyncSummaryResourceWithStreamingResponse:
        return AsyncSummaryResourceWithStreamingResponse(self._inference.summary)

    @cached_property
    def timeseries_groups(self) -> AsyncTimeseriesGroupsResourceWithStreamingResponse:
        return AsyncTimeseriesGroupsResourceWithStreamingResponse(self._inference.timeseries_groups)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["BulkListParams"]


class BulkListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    action_type: Literal["MOVE", "RELEASE"]

    page: int
    """Current page within paginated list of results."""

    per_page: int
    """The number of results per page. Maximum value is 1000."""

    status: Literal["PENDING", "DISCOVERING", "PROCESSING", "COMPLETED", "FAILED", "CANCELLED", "SKIPPED"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["RecipientCreateParams"]


class RecipientCreateParams(TypedDict, total=False):
    path_account_id: Required[Annotated[str, PropertyInfo(alias="account_id")]]
    """Account identifier."""

    body_account_id: Annotated[str, PropertyInfo(alias="account_id")]
    """Deprecated alias for `recipient_account_id`.

    Use `recipient_account_id` instead. The body field collided with the URL path
    parameter of the same name, which prevented SDK generators from distinguishing
    the source account (in the URL) from the recipient account (in the body). Both
    names will continue to be accepted until 2027-05-26 (see `x-sunset`).
    """

    organization_id: str
    """Organization identifier."""

    recipient_account_id: str
    """The account that will receive the share."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["VersionListParams"]


class VersionListParams(TypedDict, total=False):
    account_id: Required[str]

    page: float

    per_page: float



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncSinglePage, AsyncSinglePage
from ....._base_client import AsyncPaginator, make_request_options
from .....types.logs.log_explorer.datasets.available_dataset import AvailableDataset

__all__ = ["AvailableResource", "AsyncAvailableResource"]


class AvailableResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AvailableResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AvailableResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AvailableResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AvailableResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[AvailableDataset]:
        """Returns all dataset types that this account or zone can create.

        Each entry
        includes the dataset schema and timestamp field.

        The schema shows all possible fields for a dataset. However, not all fields may
        be available for your account or zone. When creating or updating a dataset, only
        fields available to your account or zone can be enabled. If you request a field
        that is not available, you will receive an error.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logs/explorer/datasets/available",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncSinglePage[AvailableDataset],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=AvailableDataset,
        )


class AsyncAvailableResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAvailableResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAvailableResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAvailableResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAvailableResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AvailableDataset, AsyncSinglePage[AvailableDataset]]:
        """Returns all dataset types that this account or zone can create.

        Each entry
        includes the dataset schema and timestamp field.

        The schema shows all possible fields for a dataset. However, not all fields may
        be available for your account or zone. When creating or updating a dataset, only
        fields available to your account or zone can be enabled. If you request a field
        that is not available, you will receive an error.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logs/explorer/datasets/available",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncSinglePage[AvailableDataset],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=AvailableDataset,
        )


class AvailableResourceWithRawResponse:
    def __init__(self, available: AvailableResource) -> None:
        self._available = available

        self.list = to_raw_response_wrapper(
            available.list,
        )


class AsyncAvailableResourceWithRawResponse:
    def __init__(self, available: AsyncAvailableResource) -> None:
        self._available = available

        self.list = async_to_raw_response_wrapper(
            available.list,
        )


class AvailableResourceWithStreamingResponse:
    def __init__(self, available: AvailableResource) -> None:
        self._available = available

        self.list = to_streamed_response_wrapper(
            available.list,
        )


class AsyncAvailableResourceWithStreamingResponse:
    def __init__(self, available: AsyncAvailableResource) -> None:
        self._available = available

        self.list = async_to_streamed_response_wrapper(
            available.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .prefix_bindings import (
    PrefixBindingsResource,
    AsyncPrefixBindingsResource,
    PrefixBindingsResourceWithRawResponse,
    AsyncPrefixBindingsResourceWithRawResponse,
    PrefixBindingsResourceWithStreamingResponse,
    AsyncPrefixBindingsResourceWithStreamingResponse,
)
from .regional_services import (
    RegionalServicesResource,
    AsyncRegionalServicesResource,
    RegionalServicesResourceWithRawResponse,
    AsyncRegionalServicesResourceWithRawResponse,
    RegionalServicesResourceWithStreamingResponse,
    AsyncRegionalServicesResourceWithStreamingResponse,
)

__all__ = [
    "PrefixBindingsResource",
    "AsyncPrefixBindingsResource",
    "PrefixBindingsResourceWithRawResponse",
    "AsyncPrefixBindingsResourceWithRawResponse",
    "PrefixBindingsResourceWithStreamingResponse",
    "AsyncPrefixBindingsResourceWithStreamingResponse",
    "RegionalServicesResource",
    "AsyncRegionalServicesResource",
    "RegionalServicesResourceWithRawResponse",
    "AsyncRegionalServicesResourceWithRawResponse",
    "RegionalServicesResourceWithStreamingResponse",
    "AsyncRegionalServicesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AppGetParams"]


class AppGetParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    page_no: int
    """The page number from which you want your page search results to be displayed."""

    per_page: int
    """Number of results per page."""

    search: str
    """Search string that matches apps by name."""

    sort_order: Literal["ASC", "DESC"]
    """Sort order for apps by creation time."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .inspect import (
    InspectResource,
    AsyncInspectResource,
    InspectResourceWithRawResponse,
    AsyncInspectResourceWithRawResponse,
    InspectResourceWithStreamingResponse,
    AsyncInspectResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["SPFResource", "AsyncSPFResource"]


class SPFResource(SyncAPIResource):
    @cached_property
    def inspect(self) -> InspectResource:
        return InspectResource(self._client)

    @cached_property
    def with_raw_response(self) -> SPFResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SPFResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SPFResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SPFResourceWithStreamingResponse(self)


class AsyncSPFResource(AsyncAPIResource):
    @cached_property
    def inspect(self) -> AsyncInspectResource:
        return AsyncInspectResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSPFResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSPFResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSPFResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSPFResourceWithStreamingResponse(self)


class SPFResourceWithRawResponse:
    def __init__(self, spf: SPFResource) -> None:
        self._spf = spf

    @cached_property
    def inspect(self) -> InspectResourceWithRawResponse:
        return InspectResourceWithRawResponse(self._spf.inspect)


class AsyncSPFResourceWithRawResponse:
    def __init__(self, spf: AsyncSPFResource) -> None:
        self._spf = spf

    @cached_property
    def inspect(self) -> AsyncInspectResourceWithRawResponse:
        return AsyncInspectResourceWithRawResponse(self._spf.inspect)


class SPFResourceWithStreamingResponse:
    def __init__(self, spf: SPFResource) -> None:
        self._spf = spf

    @cached_property
    def inspect(self) -> InspectResourceWithStreamingResponse:
        return InspectResourceWithStreamingResponse(self._spf.inspect)


class AsyncSPFResourceWithStreamingResponse:
    def __init__(self, spf: AsyncSPFResource) -> None:
        self._spf = spf

    @cached_property
    def inspect(self) -> AsyncInspectResourceWithStreamingResponse:
        return AsyncInspectResourceWithStreamingResponse(self._spf.inspect)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["OAuthClientCreateParams"]


class OAuthClientCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    client_name: Required[str]
    """Human-readable name of the OAuth client."""

    grant_types: Required[List[Literal["authorization_code", "refresh_token"]]]
    """Array of OAuth grant types the client is allowed to use.

    `authorization_code` is required; `refresh_token` may be included optionally.
    """

    redirect_uris: Required[SequenceNotStr[str]]
    """Array of allowed redirect URIs for the client."""

    response_types: Required[List[Literal["token", "id_token", "code"]]]
    """Array of OAuth response types the client is allowed to use."""

    scopes: Required[SequenceNotStr[str]]
    """Array of OAuth scopes the client is allowed to request.

    Colon-delimited scopes are not accepted. Dot-delimited scopes are validated
    against available OAuth API scopes; simple identity scopes are allowed. Protocol
    scopes `offline_access` and `openid` are added or removed automatically based on
    `grant_types` and `response_types`.
    """

    token_endpoint_auth_method: Required[Literal["none", "client_secret_basic", "client_secret_post"]]
    """The authentication method the client uses at the token endpoint."""

    allowed_cors_origins: SequenceNotStr[str]
    """Array of allowed CORS origins."""

    client_uri: str
    """URL of the home page of the client."""

    logo_uri: str
    """URL of the client's logo."""

    policy_uri: str
    """URL that points to a privacy policy document."""

    post_logout_redirect_uris: SequenceNotStr[str]
    """Array of allowed post-logout redirect URIs."""

    tos_uri: str
    """URL that points to a terms of service document."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .spf import (
    SPFResource,
    AsyncSPFResource,
    SPFResourceWithRawResponse,
    AsyncSPFResourceWithRawResponse,
    SPFResourceWithStreamingResponse,
    AsyncSPFResourceWithStreamingResponse,
)
from .email_auth import (
    EmailAuthResource,
    AsyncEmailAuthResource,
    EmailAuthResourceWithRawResponse,
    AsyncEmailAuthResourceWithRawResponse,
    EmailAuthResourceWithStreamingResponse,
    AsyncEmailAuthResourceWithStreamingResponse,
)
from .dmarc_reports import (
    DMARCReportsResource,
    AsyncDMARCReportsResource,
    DMARCReportsResourceWithRawResponse,
    AsyncDMARCReportsResourceWithRawResponse,
    DMARCReportsResourceWithStreamingResponse,
    AsyncDMARCReportsResourceWithStreamingResponse,
)

__all__ = [
    "DMARCReportsResource",
    "AsyncDMARCReportsResource",
    "DMARCReportsResourceWithRawResponse",
    "AsyncDMARCReportsResourceWithRawResponse",
    "DMARCReportsResourceWithStreamingResponse",
    "AsyncDMARCReportsResourceWithStreamingResponse",
    "SPFResource",
    "AsyncSPFResource",
    "SPFResourceWithRawResponse",
    "AsyncSPFResourceWithRawResponse",
    "SPFResourceWithStreamingResponse",
    "AsyncSPFResourceWithStreamingResponse",
    "EmailAuthResource",
    "AsyncEmailAuthResource",
    "EmailAuthResourceWithRawResponse",
    "AsyncEmailAuthResourceWithRawResponse",
    "EmailAuthResourceWithStreamingResponse",
    "AsyncEmailAuthResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from ....._utils import PropertyInfo
from ....._models import BaseModel

__all__ = ["MessageListResponse", "ActionParams", "ActionParamsMove", "ActionParamsRelease"]


class ActionParamsMove(BaseModel):
    client_recipient: str

    destination: Literal["Inbox", "JunkEmail", "DeletedItems", "RecoverableItemsDeletions", "RecoverableItemsPurges"]

    type: Literal["MOVE"]

    expected_disposition: Optional[
        Literal[
            "MALICIOUS",
            "MALICIOUS-BEC",
            "SUSPICIOUS",
            "SPOOF",
            "SPAM",
            "BULK",
            "ENCRYPTED",
            "EXTERNAL",
            "UNKNOWN",
            "NONE",
        ]
    ] = None


class ActionParamsRelease(BaseModel):
    client_recipient: str

    type: Literal["RELEASE"]


ActionParams: TypeAlias = Annotated[Union[ActionParamsMove, ActionParamsRelease], PropertyInfo(discriminator="type")]


class MessageListResponse(BaseModel):
    action_params: ActionParams

    action_type: Literal["MOVE", "RELEASE"]

    created_at: datetime

    message_id: str

    postfix_id: str

    retry_count: int

    status: Literal["PENDING", "DISCOVERING", "PROCESSING", "COMPLETED", "FAILED", "CANCELLED", "SKIPPED"]

    alert_id: Optional[str] = None

    email_message_id: Optional[str] = None

    processed_at: Optional[datetime] = None

    retry_after: Optional[datetime] = None
    """When to retry the action if it failed"""

    status_message: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr

__all__ = [
    "RuleEditParams",
    "Position",
    "PositionAPIShieldIndex",
    "PositionAPIShieldBefore",
    "PositionAPIShieldAfter",
    "Selector",
    "SelectorExclude",
    "SelectorInclude",
]


class RuleEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    action: Literal["log", "block"]
    """
    Action to take on requests that match operations included in `selector` and fail
    `expression`.
    """

    description: str
    """A human-readable description that gives more details than `title`."""

    enabled: bool
    """Toggle rule on or off."""

    expression: str
    """Rule expression.

    Requests that fail to match this expression will be subject to `action`.

    For details on expressions, see the
    [Cloudflare Docs](https://developers.cloudflare.com/api-shield/security/jwt-validation/).
    """

    position: Position
    """Update rule order among zone rules."""

    selector: Selector
    """Select operations covered by this rule.

    For details on selectors, see the
    [Cloudflare Docs](https://developers.cloudflare.com/api-shield/security/jwt-validation/).
    """

    title: str
    """A human-readable name for the rule."""


class PositionAPIShieldIndex(TypedDict, total=False):
    index: Required[int]
    """Move rule to this position"""


class PositionAPIShieldBefore(TypedDict, total=False):
    """Move rule to after rule with ID."""

    before: str
    """Move rule to before rule with this ID."""


class PositionAPIShieldAfter(TypedDict, total=False):
    """Move rule to before rule with ID."""

    after: str
    """Move rule to after rule with this ID."""


Position: TypeAlias = Union[PositionAPIShieldIndex, PositionAPIShieldBefore, PositionAPIShieldAfter]


class SelectorExclude(TypedDict, total=False):
    operation_ids: SequenceNotStr[str]
    """Excluded operation IDs."""


class SelectorInclude(TypedDict, total=False):
    host: SequenceNotStr[str]
    """Included hostnames."""


class Selector(TypedDict, total=False):
    """Select operations covered by this rule.

    For details on selectors, see the [Cloudflare Docs](https://developers.cloudflare.com/api-shield/security/jwt-validation/).
    """

    exclude: Optional[Iterable[SelectorExclude]]
    """Ignore operations that were otherwise included by `include`."""

    include: Optional[Iterable[SelectorInclude]]
    """Select all matching operations."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from ..._models import BaseModel

__all__ = ["RegionGetResponse"]


class RegionGetResponse(BaseModel):
    id: str

    created_on: datetime

    modified_on: datetime

    name: str

    region_key: str

    version: int

    version_created_on: datetime



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["RuleUpdateParams"]


class RuleUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Unique identifier linked to an account."""

    description: str

    match: str
    """The wirefilter expression to match."""

    name: str
    """The name of the Rule."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .available_list import AvailableList as AvailableList
from .available_dataset import AvailableDataset as AvailableDataset



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["OAuthScopeListResponse"]


class OAuthScopeListResponse(BaseModel):
    """An available OAuth scope that can be assigned to an OAuth client."""

    id: str
    """
    The scope label to use in the scopes array when creating or updating an OAuth
    client.
    """

    name: str
    """Human-readable name of the OAuth scope."""

    category: Optional[str] = None
    """Category for grouping scopes in the UI."""

    scopes: Optional[List[str]] = None
    """
    The underlying resource scopes (Bach scopes) that define which resources this
    OAuth scope can act upon.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from .apps.apps import (
    AppsResource,
    AsyncAppsResource,
    AppsResourceWithRawResponse,
    AsyncAppsResourceWithRawResponse,
    AppsResourceWithStreamingResponse,
    AsyncAppsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["FlagshipResource", "AsyncFlagshipResource"]


class FlagshipResource(SyncAPIResource):
    @cached_property
    def apps(self) -> AppsResource:
        return AppsResource(self._client)

    @cached_property
    def with_raw_response(self) -> FlagshipResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FlagshipResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FlagshipResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FlagshipResourceWithStreamingResponse(self)


class AsyncFlagshipResource(AsyncAPIResource):
    @cached_property
    def apps(self) -> AsyncAppsResource:
        return AsyncAppsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFlagshipResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFlagshipResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFlagshipResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFlagshipResourceWithStreamingResponse(self)


class FlagshipResourceWithRawResponse:
    def __init__(self, flagship: FlagshipResource) -> None:
        self._flagship = flagship

    @cached_property
    def apps(self) -> AppsResourceWithRawResponse:
        return AppsResourceWithRawResponse(self._flagship.apps)


class AsyncFlagshipResourceWithRawResponse:
    def __init__(self, flagship: AsyncFlagshipResource) -> None:
        self._flagship = flagship

    @cached_property
    def apps(self) -> AsyncAppsResourceWithRawResponse:
        return AsyncAppsResourceWithRawResponse(self._flagship.apps)


class FlagshipResourceWithStreamingResponse:
    def __init__(self, flagship: FlagshipResource) -> None:
        self._flagship = flagship

    @cached_property
    def apps(self) -> AppsResourceWithStreamingResponse:
        return AppsResourceWithStreamingResponse(self._flagship.apps)


class AsyncFlagshipResourceWithStreamingResponse:
    def __init__(self, flagship: AsyncFlagshipResource) -> None:
        self._flagship = flagship

    @cached_property
    def apps(self) -> AsyncAppsResourceWithStreamingResponse:
        return AsyncAppsResourceWithStreamingResponse(self._flagship.apps)
